package rest

import (
	"fmt"
	"time"

	"github.com/araddon/dateparse"
)

var (
	fieldConverters = make(map[string]*FieldConverter)
)

type FieldConverter struct {
	Pack   func(any) any
	Unpack func(any) (any, error)
}

func RegisterFieldConverter(name string, converter *FieldConverter) {
	fieldConverters[name] = converter
}

var (
	fcTime = &FieldConverter{
		Pack: func(v any) any {
			t := v.(time.Time)
			return t.Format(time.RFC3339)
		},
		Unpack: func(val any) (any, error) {
			ts, check := val.(string)
			if !check {
				return nil, fmt.Errorf("expected string")
			}
			//layout := "02.01.2006 15:04"
			//parsedTime, err := time.Parse(layout, ts)
			parsedTime, err := dateparse.ParseAny(ts)
			if err != nil {
				return nil, err
			}
			return parsedTime, nil
		},
	}
)

func init() {
	RegisterFieldConverter("time.Time", fcTime)
}
